@@ -33,6 +33,8 @@ Where type is one of:
     -tlxs     Translational stop sites (unique stop codons)
     -junc5    Splice junction 5' donor
     -junc3    Splice junction 3' acceptor
+    -utr5     5' UTR (including introns)
+    -utr3     3' UTR (including introns)
 
     -promoter length    Promoter region from the gene [length] upstream of TSS
                         
@@ -59,7 +61,7 @@ def gtf_junc_5_tobed(gtf, out=sys.stdout):
                         continue
 
                     if not end in sites:
-                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, end, end + 1, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, end, end + 1, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                         sites.add(end)
             else:
                 for j, (start, end) in enumerate(txscr.exons):
@@ -67,7 +69,7 @@ def gtf_junc_5_tobed(gtf, out=sys.stdout):
                         continue
 
                     if not start in sites:
-                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start - 1, start, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start - 1, start, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                         sites.add(end)
 
 
@@ -81,7 +83,7 @@ def gtf_junc_3_tobed(gtf, out=sys.stdout):
                         continue
 
                     if not end in sites:
-                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, end, end + 1, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, end, end + 1, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                         sites.add(end)
             else:
                 for j, (start, end) in enumerate(txscr.exons):
@@ -89,13 +91,13 @@ def gtf_junc_3_tobed(gtf, out=sys.stdout):
                         continue
 
                     if not start in sites:
-                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start - 1, start, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start - 1, start, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                         sites.add(end)
 
 
 def gtf_genes_tobed(gtf, out=sys.stdout):
     for gene in gtf.genes:
-        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, gene.start, gene.end, gene.gene_name, 0, gene.strand]]))
+        out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, gene.start, gene.end, gene.gene_name if gene.gene_name else gene.gid, 0, gene.strand]]))
 
 
 def gtf_promoter_tobed(gtf, promoter_up, promoter_down, out=sys.stdout):
@@ -104,11 +106,11 @@ def gtf_promoter_tobed(gtf, promoter_up, promoter_down, out=sys.stdout):
         for i, txscr in enumerate(gene.transcripts):
             if gene.strand == '+':
                 if not txscr.start in sites:
-                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start - promoter_up, txscr.start + promoter_down, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start - promoter_up, txscr.start + promoter_down, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                     sites.add(txscr.start)
             else:
                 if not txscr.end in sites:
-                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.end - promoter_down, txscr.end + promoter_up, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.end - promoter_down, txscr.end + promoter_up, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                     sites.add(txscr.end)
 
 def gtf_tss_tobed(gtf, out=sys.stdout):
@@ -117,11 +119,11 @@ def gtf_tss_tobed(gtf, out=sys.stdout):
         for i, txscr in enumerate(gene.transcripts):
             if gene.strand == '+':
                 if not txscr.start in sites:
-                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start, txscr.start + 3, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start, txscr.start + 3, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                     sites.add(txscr.start)
             else:
                 if not txscr.end in sites:
-                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.end - 3, txscr.end, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.end - 3, txscr.end, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                     sites.add(txscr.end)
 
 
@@ -131,11 +133,11 @@ def gtf_txs_tobed(gtf, out=sys.stdout):
         for i, txscr in enumerate(gene.transcripts):
             if gene.strand == '-':
                 if not txscr.start in sites:
-                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start, txscr.start + 3, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start, txscr.start + 3, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                     sites.add(txscr.start)
             else:
                 if not txscr.end in sites:
-                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.end - 3, txscr.end, '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.end - 3, txscr.end, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                     sites.add(txscr.end)
 
 
@@ -147,20 +149,54 @@ def gtf_tlss_tobed(gtf, out=sys.stdout):
             if not txscr.has_cds:
                 continue
             if not txscr.start_codon in sites:
-                out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start_codon[0], txscr.start_codon[1], '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
+                out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start_codon[0], txscr.start_codon[1], '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
                 sites.add(txscr.start_codon)
 
 
+def gtf_utr5_tobed(gtf, out=sys.stdout):
+    'Outputs all 5\'UTR regions (from all transcripts)'
+    for gene in gtf.genes:
+        sites = set()
+        for i, txscr in enumerate(gene.transcripts):
+            if not txscr.has_cds:
+                continue
+            if gene.strand == '+':
+                if not (txscr.start,txscr.start_codon) in sites:
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start, txscr.start_codon[0], '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    sites.add((txscr.start,txscr.start_codon))
+            else:
+                if not (txscr.end,txscr.start_codon) in sites:
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start_codon[1]+1, txscr.end, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    sites.add((txscr.end,txscr.start_codon))
+
+
+def gtf_utr3_tobed(gtf, out=sys.stdout):
+    'Outputs all 3\'UTR regions (from all transcripts)'
+    for gene in gtf.genes:
+        sites = set()
+        for i, txscr in enumerate(gene.transcripts):
+            if not txscr.has_cds:
+                continue
+
+            if gene.strand == '+':
+                if not (txscr.stop_codon,txscr.end) in sites:
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.stop_codon[1]+1, txscr.end, '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    sites.add((txscr.start,txscr.start_codon))
+            else:
+                if not (txscr.start, txscr.stop_codon) in sites:
+                    out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.start, txscr.stop_codon[0], '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
+                    sites.add((txscr.start, txscr.stop_codon))
+
 def gtf_tlxs_tobed(gtf, out=sys.stdout):
-    'Outputs all exons (from all transcripts)'
+    'Outputs all translational stop sites (from all transcripts)'
     for gene in gtf.genes:
         sites = set()
         for i, txscr in enumerate(gene.transcripts):
             if not txscr.has_cds:
                 continue
             if not txscr.stop_codon in sites:
-                out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.stop_codon[0], txscr.stop_codon[1], '%s.%s' % (gene.gene_name, i), 0, gene.strand]]))
-                sites.add(txscr.start_codon)
+                out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, txscr.stop_codon[0], txscr.stop_codon[1], '%s/%s' % (gene.gene_name, i), 0, gene.strand]]))
+                sites.add(txscr.stop_codon)
 
 
 def gtf_exons_tobed(gtf, out=sys.stdout):
@@ -171,7 +207,7 @@ def gtf_exons_tobed(gtf, out=sys.stdout):
             exons.update(txscr.exons)
 
         for i, (start, end) in enumerate(sorted(exons)):
-            out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start, end, '%s.e%s' % (gene.gene_name, i + 1), 0, gene.strand]]))
+            out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start, end, '%s/e%s' % (gene.gene_name, i + 1), 0, gene.strand]]))
 
 
 def gtf_introns_tobed(gtf, out=sys.stdout):
@@ -181,13 +217,13 @@ def gtf_introns_tobed(gtf, out=sys.stdout):
         
         for txscr in gene.transcripts:
             last = None
-            for start, end in txscr.exons:
+            for start, end in sorted(txscr.exons):
                 if last:
                     introns.add((last, start))
                 last = end
 
         for i, (start, end) in enumerate(sorted(introns)):
-            out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start, end, '%s.i.%s' % (gene.gene_name, i + 1), 0, gene.strand]]))
+            out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start, end, '%s/i%s' % (gene.gene_name, i + 1), 0, gene.strand]]))
 
 
 def gtf_regions_tobed(gtf, out=sys.stdout):
@@ -197,7 +233,7 @@ def gtf_regions_tobed(gtf, out=sys.stdout):
             source_count = 0
             for n in names.split(','):
                 source_count += 1
-            out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start, end, '%s.%s.%s' % (gene.gene_name, 'const' if const else 'alt', i), source_count, gene.strand]]))
+            out.write('%s\n' % '\t'.join([str(x) for x in [gene.chrom, start, end, '%s/%s.%s' % (gene.gene_name, 'const' if const else 'alt', i), source_count, gene.strand]]))
 
 
 if __name__ == '__main__':
@@ -211,6 +247,8 @@ if __name__ == '__main__':
     tlxs = False
     junc_5 = False
     junc_3 = False
+    utr_5 = False
+    utr_3 = False
     promoter = False
     promoter_up = 0
     promoter_down = 0
@@ -244,6 +282,10 @@ if __name__ == '__main__':
             txs = True
         elif arg == '-tlxs':
             tlxs = True
+        elif arg == '-utr5':
+            utr_5 = True
+        elif arg == '-utr3':
+            utr_3 = True
         elif arg == '-junc5':
             junc_5 = True
         elif arg == '-junc3':
@@ -255,7 +297,7 @@ if __name__ == '__main__':
             filename = arg
 
     i = 0
-    for arg in [genes, exons, introns, regions, tss, tlss, txs, tlxs, junc_5, junc_3, promoter]:
+    for arg in [genes, exons, introns, regions, tss, tlss, txs, tlxs, utr_5, utr_3, junc_5, junc_3, promoter]:
         if arg:
             i += 1
 
@@ -286,6 +328,10 @@ if __name__ == '__main__':
         gtf_txs_tobed(gtf)
     elif tlxs:
         gtf_tlxs_tobed(gtf)
+    elif utr_5:
+        gtf_utr5_tobed(gtf)
+    elif utr_3:
+        gtf_utr3_tobed(gtf)
     elif junc_5:
         gtf_junc_5_tobed(gtf)
     elif junc_3:
